@@ -1,5 +1,6 @@
-/* Copyright (C) 2000, 2001 Stefan Buehler <sbuehler@uni-bremen.de>
-                            Axel von Engeln <engeln@uni-bremen.de>
+/* Copyright (C) 2000-2008
+   Stefan Buehler <sbuehler@ltu.se>
+   Axel von Engeln <engeln@uni-bremen.de>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -19,10 +20,10 @@
 /** \file
     Declarations required for the calculation of absorption coefficients.
 
-    \author Stefan Buehler
-*/
+    This is the file from arts-1-0, back-ported to arts-1-1.
 
-// This file contains 
+    \author Stefan Buehler, Axel von Engeln
+*/
 
 #ifndef absorption_h
 #define absorption_h
@@ -32,6 +33,7 @@
 #include "array.h"
 #include "mystring.h"
 #include "make_array.h"
+#include "messages.h"
 
 /** The type that is used to store pointers to lineshape
     functions.  */
@@ -52,7 +54,10 @@ class LineshapeRecord{
 public:
 
   /** Default constructor. */
-  LineshapeRecord(){};
+  LineshapeRecord() : mname(),
+                      mdescription(),
+                      mfunction()
+  { /* Nothing to do here. */ }
 
   /** Initializing constructor, used to build the lookup table. */
   LineshapeRecord(const String& name,
@@ -93,7 +98,10 @@ class LineshapeNormRecord{
 public:
 
   /** Default constructor. */
-  LineshapeNormRecord(){};
+  LineshapeNormRecord() : mname(),
+                          mdescription(),
+                          mfunction()
+  { /* Nothing to do here. */ }
 
   /** Initializing constructor, used to build the lookup table. */
   LineshapeNormRecord(const String& name,
@@ -124,7 +132,10 @@ class LineshapeSpec{
 public:
 
   /** Default constructor. */
-  LineshapeSpec(){};
+  LineshapeSpec() : mind_ls(-1),
+                    mind_lsn(-1),
+                    mcutoff(0.)
+  { /* Nothing to do here. */ }
 
   /** Initializing constructor. */
   LineshapeSpec(const Index&    ind_ls,
@@ -157,6 +168,8 @@ private:
   Numeric mcutoff;
 };
 
+ostream& operator<< (ostream& os, const LineshapeSpec& lsspec);
+
 /** Holds a list of lineshape specifications: function, normalization, cutoff.
     \author Axel von Engeln */
 typedef Array<LineshapeSpec> ArrayOfLineshapeSpec;
@@ -169,7 +182,14 @@ class IsotopeRecord{
 public:
 
   /** Default constructor. Needed by make_array. */
-  IsotopeRecord() { /* Nothing to do here */ }
+  IsotopeRecord() : mname(),
+                    mabundance(0.),
+                    mmass(0.),
+                    mmytrantag(-1),
+                    mhitrantag(-1),
+                    mjpltags(),
+                    mqcoeff()
+  { /* Nothing left to do here. */ }
 
   /** Copy constructor. We need this, since operator= does not work
       correctly for Arrays. (Target Array has to be resized first.) */
@@ -179,7 +199,8 @@ public:
     mmass(x.mmass),
     mmytrantag(x.mmytrantag),
     mhitrantag(x.mhitrantag),
-    mjpltags(x.mjpltags)
+    mjpltags(x.mjpltags),
+    mqcoeff()
   { /* Nothing left to do here. */ }
 
   /** Constructor that sets the values. */
@@ -194,7 +215,8 @@ public:
     mmass(mass),
     mmytrantag(mytrantag),
     mhitrantag(hitrantag),
-    mjpltags(jpltags)
+    mjpltags(jpltags),
+    mqcoeff()
   {
     // With Matpack, initialization of mjpltags from jpltags should now work correctly.
 
@@ -217,9 +239,9 @@ public:
   /** Mass of the isotope. (In unified atomic mass units u)
       If I understand this correctly this is the same as g/mol. */
   const Numeric&      Mass()         const { return mmass;    }
-  /** MYTRAN2 tag numers for all isotopes. -1 means not included. */
+  /** MYTRAN2 tag numbers for all isotopes. -1 means not included. */
   const Index&          MytranTag()    const { return mmytrantag;    }
-  /** HITRAN-96 tag numers for all isotopes. -1 means not included. */
+  /** HITRAN-96 tag numbers for all isotopes. -1 means not included. */
   const Index&          HitranTag()    const { return mhitrantag;    }
   /** JPL tag numbers for all isotopes. Empty array means not included. There
       can be more than one JPL tag for an isotopic species, because in
@@ -252,11 +274,12 @@ public:
     Numeric qtemp =
       CalculatePartitionFctAtTemp( actual_temperature    );
 
-    if ( qtemp <= 0. ) 
+    if ( qtemp < 0. ) 
       {
         ostringstream os;
-        os << "Negative value of partition function was obtained for "
-           << mname << ".\nProbably caused by low or negative temperature.";
+        os << "Partition function of "
+           << "Isotope = " << mname
+           << "is unknown.";
         throw runtime_error(os.str());
       }
     return qcoeff_at_t_ref / qtemp;
@@ -285,7 +308,9 @@ class SpeciesRecord{
 public:
 
   /** Default constructor. */
-  SpeciesRecord(){}  ;
+  SpeciesRecord() : mname(),
+                    mdegfr(-1),
+                    misotope() { /* Nothing to do here */ }
   
   /** The constructor used in define_species_data. */
   SpeciesRecord(const char name[],
@@ -348,28 +373,40 @@ private:
   Array<IsotopeRecord> misotope;
 };
 
-/** Spectral line catalog data. Here is a description of the ARTS
-    catalogue format:
+/** Spectral line catalog data. 
 
-    To keep track of changes in the catalog format, every catalogue
-    file must start with ARTSCAT-x. The current version is x=2. Files
-    with different or missing version will be rejected. The current
-    version is stored in the private member variable mversion. It can
-    be read with member function Version, which returns a String
-    `ARTSCAT-x'. 
+    Below is a description of the ARTS catalogue format.
+    The file starts with the usual XML header:
 
-    After the version tag (ARTSCAT-x), ARTS outputs the number of
-    lines when catalogue files are written. This number is not used by
-    reading routines, though.
+    \verbatim
+    <?xml version="1.0"?>
+    <arts format="ascii" version="1">
+    <ArrayOfLineRecord version="ARTSCAT-3" nelem="8073">
+    \endverbatim
 
-    The line catalogue should not have any fixed column widths because the
-    precision of the parameters should not be limited by the format.  The
-    catalogue can then be stored as binary or ASCII. In the ASCII version
-    the columns are separated by one or more blanks. The line format is
-    then specified by only the order and the units of the columns. As
-    the catalogue entry for each transition can be quite long, it can be
-    broken across lines in the ASCII file. Each new transition is marked
-    with a `@' character.
+    The ARTSCAT version number is there to keep track of catalogue format changes.
+    The "nelem" tag contains the total number of lines in the file.
+
+    The file ends with the usual XML closing tags:
+
+    \verbatim
+    </ArrayOfLineRecord>
+    </arts>
+    \endverbatim
+
+    In-between the header and the footer are the actual spectroscopic
+    data. Each new entry, corresponding to one spectral line, starts
+    with the `@' character.
+
+    The line catalogue should not have any fixed column widths because
+    the precision of the parameters should not be limited by the
+    format.  The catalogue can then be stored as binary or ASCII. In
+    the ASCII version the columns are separated by one or more
+    blanks. The line format is then specified by only the order and
+    the units of the columns. As the catalogue entry for each
+    transition can be quite long, it can be broken across lines in the
+    ASCII file. That is why each new transition is marked with a `@'
+    character.
 
     The first column will contain the species and isotope, following
     the naming scheme described below.  Scientific notation is
@@ -384,16 +421,16 @@ private:
     \verbatim
     Col  Variable                Label    Unit     Comment
     ------------------------------------------------------------------      
-     0   `@'                         ENTRY        -     marks start of entry
-     1   name                         NAME        -     e.g. O3-666
-     2   center frequency                F       Hz     e.g. 501.12345e9 
+     0   `@'                         ENTRY        -    marks start of entry
+     1   name                         NAME        -    e.g. O3-666
+     2   center frequency                F       Hz    e.g. 501.12345e9 
      3   pressure shift of F           PSF    Hz/Pa    
-     4   line intensity                 I0   m^2/Hz     per isotope, not per species
+     4   line intensity                 I0    Hz*m^2   per isotope, not per species
      5   reference temp. for I0       T_I0        K
      6   lower state energy           ELOW        J    
-     7   air broadened width          AGAM    Hz/Pa     values around 20 GHz/Pa
+     7   air broadened width          AGAM    Hz/Pa    values around 20000 Hz/Pa
      8   self broadened width         SGAM    Hz/Pa
-     9   AGAM temp. exponent          NAIR        -     values around .5
+     9   AGAM temp. exponent          NAIR        -    values around .5
     10   SGAM temp. exponent         NSELF        - 
     11   ref. temp. for AGAM, SGAM   T_GAM        K
     12   number of aux. parameters   N_AUX        -
@@ -415,17 +452,23 @@ private:
 
     A valid ARTS line file would be:
     \verbatim
-    ARTSCAT-2 2
-    @ CH4-211 1011349857.063 0 2.96070344144819e-27 296 2183.6851 13314.2468393782 21302.7949430052 0.75 0.75 296 0
-    @ O3-666 1088246622.54 0 2.82913939200384e-22 296 522.5576 21361.9693734024 27723.2206411054 0.76 0.76 296 0
+    <?xml version="1.0"?>
+    <arts format="ascii" version="1">
+    <ArrayOfLineRecord version="ARTSCAT-3" nelem="2">
+    @ O3-676 80015326542.0992 0 3.70209114155527e-19 296 7.73661776567701e-21 21480.3182341969
+    28906.7092490501 0.76 0.76 296 0 300000 0.1 0.1 0.1 0.2 -1 -1.24976056865038e-11
+    @ O3-676 80015476438.3282 0 3.83245786810611e-19 296 7.73661577919822e-21 21480.3182341969
+    28906.7092490501 0.76 0.76 296 0 300000 0.1 0.1 0.1 0.2 -1 -1.24975822742335e-11
+    </ArrayOfLineRecord>
+    </arts>
     \endverbatim
 
     Some species need special parameters that are not needed by other
     species (for example overlap coefficients for O2). In the case of
     oxygen two parameters are sufficient to describe the overlap, but
     other species, e.g., methane, may need more coefficients. The
-    default for \texttt{N\_AUX} is zero. In that case, no further
-    \texttt{AUX} fields are present. [FIXME: Check Oxygen.]
+    default for \c N_AUX is zero. In that case, no further
+    \c AUX fields are present. [FIXME: Check Oxygen.]
 
     The names of the private members and public access functions of
     this data structure follow the above table. The only difference is
@@ -441,13 +484,14 @@ public:
   /** Make the helper function linesElowToJoule a friend, so that it
       can change the lower state energy. FIXME: Remove this when we
       don't need the helper function anymore. */
-  friend void linesElowToJoule(Array<LineRecord>& lines);
+  friend void linesElowToJoule(Array<LineRecord>& abs_lines);
 
   /** Default constructor. Initialize to default values. The indices
       are initialized to large numbers, so that we at least get range
       errors when we try to used un-initialized data. */
   LineRecord()
-    : mspecies (1000000),
+    : mversion (3),
+      mspecies (1000000),
       misotope (1000000),
       mf       (0.     ),
       mpsf     (0.     ),
@@ -470,7 +514,7 @@ public:
  { /* Nothing to do here. */ }
 
   /** Constructor that sets all data elements explicitly. If
-      assertions are not disabled (i.e., if NDEBUG is not #defined),
+      assertions are not disabled (i.e., if NDEBUG is not \#defined),
       assert statements check that the species and isotope data
       exists. */
   LineRecord( Index                 species,
@@ -493,7 +537,8 @@ public:
               Numeric               /* dnair */,
               Numeric               /* dnself */,
               Numeric               /* dpsf */)
-    : mspecies (species    ),
+    : mversion (3),
+      mspecies (species    ),
       misotope (isotope    ),
       mf       (f          ),
       mpsf     (psf        ),
@@ -505,26 +550,36 @@ public:
       mnair    (nair       ),
       mnself   (nself      ),
       mtgam    (tgam       ), 
-      maux     (aux        )
+      maux     (aux        ),
+      mdf      (-1.    ),
+      mdi0     (-1.    ),
+      mdagam   (-1.    ),
+      mdsgam   (-1.    ),
+      mdnair   (-1.    ),
+      mdnself  (-1.    ),
+      mdpsf    (-1.    )
   {
     // Thanks to Matpack, initialization of misotope with isotope
     // should now work correctly.  
 
     // Check if this species is legal, i.e., if species and isotope
     // data exists.
-    extern Array<SpeciesRecord> species_data;
-    assert( mspecies < species_data.nelem() );
-    assert( misotope < species_data[mspecies].Isotope().nelem() );
+    ////    extern Array<SpeciesRecord> species_data;
+    //assert( mspecies < species_data.nelem() );
+    //assert( misotope < species_data[mspecies].Isotope().nelem() );
   }
 
   /** Return the version String. */
-  String Version() const
+  String VersionString() const
   {
     ostringstream os;
     os << "ARTSCAT-" << mversion;
     return os.str();
   }
-
+  
+  /** Return the version number. */
+  Index Version() const { return mversion; }
+  
   /** The index of the molecular species that this line belongs
       to. The species data can be accessed by species_data[Species()]. */
   Index Species() const { return mspecies; }
@@ -534,47 +589,11 @@ public:
       species_data[Species()].Isotope()[Isotope()].  */
   Index Isotope() const { return misotope; }
 
-  /** The full name of the species and isotope. E.g., `O3-666'. The
-      name is found by looking up the information in species_data,
-      using the species and isotope index. */
-  String Name() const {
-    // The species lookup data:
-    extern const Array<SpeciesRecord> species_data;
-    const SpeciesRecord& sr = species_data[mspecies];
-    return sr.Name() + "-" + sr.Isotope()[misotope].Name();
-  }
+  String Name() const;
 
-  /** The matching SpeciesRecord from species_data. To get at the
-      species data of a LineRecord lr, you can use:
-      \begin{enumerate}
-      \item species_data[lr.Species()]
-      \item lr.SpeciesData()
-      \end{enumerate}
-      The only advantages of the latter are that the notation is
-      slightly nicer and that you don't have to declare the external
-      variable species_data. */
-  const SpeciesRecord& SpeciesData() const {
-    // The species lookup data:
-    extern const Array<SpeciesRecord> species_data;
-    return species_data[mspecies];
-  }
+  const SpeciesRecord& SpeciesData() const;
 
-  /** The matching IsotopeRecord from species_data. The IsotopeRecord
-      is a subset of the SpeciesRecord. To get at the isotope data of
-      a LineRecord lr, you can use:
-      \begin{enumerate}
-      \item species_data[lr.Species()].Isotope()[lr.Isotope()]
-      \item lr.SpeciesData().Isotope()[lr.Isotope()]
-      \item lr.IsotopeData()
-      \end{enumerate}
-      The last option is clearly the shortest, and has the advantage
-      that you don't have to declare the external variable
-      species_data. */
-  const IsotopeRecord& IsotopeData() const {
-    // The species lookup data:
-    extern const Array<SpeciesRecord> species_data;
-    return species_data[mspecies].Isotope()[misotope];
-  }
+  const IsotopeRecord& IsotopeData() const;
 
   /** The line center frequency in <b> Hz</b>. */
   Numeric F() const     { return mf; }
@@ -667,10 +686,78 @@ public:
   /** Accuracy for SGAM temperature exponent in <b> relative value</b>: */
   Numeric dNself() const { return mdnself; }
 
- /** Accuracy for pressure shift in <b> relative value </b>: */
+  /** Accuracy for pressure shift in <b> relative value </b>: */
   Numeric dPsf() const { return mdpsf; }
 
-
+  /** Einstein A-coefficient in <b> 1/s </b>: */
+  Numeric A() const { return ma; }
+  
+  /** Upper state stat. weight: */
+  Numeric G_upper() const { return mgupper; }
+  
+  /** Lower state stat. weight: */
+  Numeric G_lower() const { return mglower; }
+  
+  /** Broadening parameter self in <b> Hz/Pa </b>: */
+  Numeric Gamma_self() const { return mgamma_self; }
+  
+  /** Broadening parameter N2 in <b> Hz/Pa </b>: */
+  Numeric Gamma_N2() const { return mgamma_n2; }
+  
+  /** Broadening parameter O2 in <b> Hz/Pa </b>: */
+  Numeric Gamma_O2() const { return mgamma_o2; }
+  
+  /** Broadening parameter H2O in <b> Hz/Pa </b>: */
+  Numeric Gamma_H2O() const { return mgamma_h2o; }
+  
+  /** Broadening parameter CO2 in <b> Hz/Pa </b>: */
+  Numeric Gamma_CO2() const { return mgamma_co2; }
+  
+  /** Broadening parameter H2 in <b> Hz/Pa </b>: */
+  Numeric Gamma_H2() const { return mgamma_h2; }
+  
+  /** Broadening parameter He in <b> Hz/Pa </b>: */
+  Numeric Gamma_He() const { return mgamma_he; }
+  
+  /** GAM temp. exponent N self: */
+  Numeric Gam_N_self() const { return mn_self; }
+  
+  /** GAM temp. exponent N N2: */
+  Numeric Gam_N_N2() const { return mn_n2; }
+  
+  /** GAM temp. exponent N O2: */
+  Numeric Gam_N_O2() const { return mn_o2; }
+  
+  /** GAM temp. exponent N H2O: */
+  Numeric Gam_N_H2O() const { return mn_h2o; }
+  
+  /** GAM temp. exponent N CO2: */
+  Numeric Gam_N_CO2() const { return mn_co2; }
+  
+  /** GAM temp. exponent N H2: */
+  Numeric Gam_N_H2() const { return mn_h2; }
+  
+  /** GAM temp. exponent N He: */
+  Numeric Gam_N_He() const { return mn_he; }
+  
+  /** F Pressure shift N2 in <b> Hz/Pa </b>: */
+  Numeric Delta_N2() const { return mdelta_n2; }
+  
+  /** F Pressure shift O2 in <b> Hz/Pa </b>: */
+  Numeric Delta_O2() const { return mdelta_o2; }
+  
+  /** F Pressure shift H2O in <b> Hz/Pa </b>: */
+  Numeric Delta_H2O() const { return mdelta_h2o; }
+  
+  /** F Pressure shift CO2 in <b> Hz/Pa </b>: */
+  Numeric Delta_CO2() const { return mdelta_co2; }
+  
+  /** F Pressure shift H2 in <b> Hz/Pa </b>: */
+  Numeric Delta_H2() const { return mdelta_h2; }
+  
+  /** F Pressure shift He in <b> Hz/Pa </b>: */
+  Numeric Delta_He() const { return mdelta_he; }
+  
   /** Read one line from a stream associated with a HITRAN 1986-2001 file. The
     HITRAN format is as follows (directly from the HITRAN documentation):
 
@@ -730,7 +817,7 @@ public:
     \return false=ok (data returned), true=eof (no data returned)
 
     \author Stefan Buehler */
-  bool ReadFromHitranStream(istream& is);
+  bool ReadFromHitranStream(istream& is, const Verbosity& verbosity);
 
 
 
@@ -803,7 +890,7 @@ public:
     \return false=ok (data returned), true=eof (no data returned)
 
     \author Stefan Buehler, Hermann Berg */
-  bool ReadFromHitran2004Stream(istream& is);
+  bool ReadFromHitran2004Stream(istream& is, const Verbosity& verbosity);
 
 
 
@@ -876,7 +963,7 @@ public:
     \date 31.10.00
     \author Axel von Engeln 
   */
-  bool ReadFromMytran2Stream(istream& is);
+  bool ReadFromMytran2Stream(istream& is, const Verbosity& verbosity);
 
 
   /** Read one line from a stream associated with a JPL file. The JPL
@@ -931,9 +1018,9 @@ public:
 
     \date 01.11.00
     \author Axel von Engeln */
-  bool ReadFromJplStream(istream& is);
+  bool ReadFromJplStream(istream& is, const Verbosity& verbosity);
 
-  /** Read one line from a stream associated with an Arts file.
+  /** Read one line from a stream associated with an ARTSCAT-3 file.
 
       Format: see Documentation of class LineRecord
 
@@ -954,12 +1041,31 @@ public:
       \date   2001-06-20
       \author Stefan Buehler
 */
-  bool ReadFromArtsStream(istream& is);
+  bool ReadFromArtscat3Stream(istream& is, const Verbosity& verbosity);
 
+  /** Read one line from a stream associated with an ARTSCAT-4 file.
+   
+   Format: see Documentation of class LineRecord
+   
+   The function attempts to read a line of data from the
+   catalogue. It returns false if it succeeds. Otherwise, if eof is
+   reached, it returns true. If an error occurs, a runtime_error is
+   thrown. When the function looks for a data line, comment lines are
+   automatically skipped.
+   
+   \param is Stream from which to read
+   \exception runtime_error Some error occured during the read
+   \return false=ok (data returned), true=eof (no data returned)
+   
+   \date   2012-02-10
+   \author Oliver Lemke
+   */
+  bool ReadFromArtscat4Stream(istream& is, const Verbosity& verbosity);
+  
 
 private:
-  // Versin number:
-  static const Index mversion = 3;
+  // Version number:
+  Index mversion;
   // Molecular species index: 
   Index mspecies;
   // Isotopic species index:
@@ -1003,6 +1109,59 @@ private:
   Numeric mdnself;
  //  Accuracy for pressure shift in %:
   Numeric mdpsf;
+  
+  //// New fields in ARTSCAT-4
+  
+  // Einstein A-coefficient in 1/s:
+  Numeric ma;
+  // Upper state stat. weight:
+  Numeric mgupper;
+  // Lower state stat. weight:
+  Numeric mglower;
+  
+  // Broadening parameter self in Hz/Pa:
+  Numeric mgamma_self;
+  // Broadening parameter N2 in Hz/Pa:
+  Numeric mgamma_n2;
+  // Broadening parameter O2 in Hz/Pa:
+  Numeric mgamma_o2;
+  // Broadening parameter H2O in Hz/Pa:
+  Numeric mgamma_h2o;
+  // Broadening parameter CO2 in Hz/Pa:
+  Numeric mgamma_co2;
+  // Broadening parameter H2 in Hz/Pa:
+  Numeric mgamma_h2;
+  // Broadening parameter He in Hz/Pa:
+  Numeric mgamma_he;
+
+  // GAM temp. exponent self:
+  Numeric mn_self;
+  // GAM temp. exponent N2:
+  Numeric mn_n2;
+  // GAM temp. exponent O2:
+  Numeric mn_o2;
+  // GAM temp. exponent H2O:
+  Numeric mn_h2o;
+  // GAM temp. exponent CO2:
+  Numeric mn_co2;
+  // GAM temp. exponent H2:
+  Numeric mn_h2;
+  // GAM temp. exponent He:
+  Numeric mn_he;
+  
+  // F Pressure shift N2 in Hz/Pa:
+  Numeric mdelta_n2;
+  // F Pressure shift O2 in Hz/Pa:
+  Numeric mdelta_o2;
+  // F Pressure shift H2O in Hz/Pa:
+  Numeric mdelta_h2o;
+  // F Pressure shift CO2 in Hz/Pa:
+  Numeric mdelta_co2;
+  // F Pressure shift H2 in Hz/Pa:
+  Numeric mdelta_h2;
+  // F Pressure shift He in Hz/Pa:
+  Numeric mdelta_he;
+  
 };
 
 // is needed to map jpl tags/arts identifier to the species/isotope data within arts
@@ -1042,7 +1201,7 @@ typedef Array< Array<LineRecord> > ArrayOfArrayOfLineRecord;
     catalogue line.
 
     \author Stefan Buehler */
-ostream& operator << (ostream& os, const LineRecord& lr);
+ostream& operator<< (ostream& os, const LineRecord& lr);
 
 
 
@@ -1052,107 +1211,22 @@ ostream& operator << (ostream& os, const LineRecord& lr);
 void define_species_map();
 
 
-
-//------------------------------< Tag Group Stuff >------------------------------
-
-/** A tag group can consist of the sum of several of these.
-
-    \author Stefan Buehler */
-class OneTag {
-public:
-  /** Default constructor. */
-  OneTag() { /* Nothing to be done here. */ }
-
-  /** Constructor from a tag definition String (Bredbeck
-      notation). For examples see member function Name(). 
-
-      \exception runtime_error The given String could not be mapped to
-      a sensible tag description. */
-  OneTag(String def); 
-
-  /** Return the full name of this tag according to Bredbeck
-      convention. Examples:
-      \verbatim
-      O3-*-*-*         : All O3 lines
-      O3-666-*-*       : All O3-666 lines
-      O3-*-500e9-501e9 : All O3 lines between 500 and 501 GHz.
-      \endverbatim */
-  String Name() const;
-    
-  /** Molecular species index. */
-  Index Species() const { return mspecies; }
-
-  /** Isotopic species index.
-      If this is equal to the number of isotopes (one more than
-      allowed) it means all isotopes of this species. */ 
-  Index Isotope() const { return misotope; }
-
-  /** The lower line center frequency in Hz.
-      If this is <0 it means no lower limit. */
-  Numeric Lf() const { return mlf; }
-
-  /** The upper line center frequency in Hz:
-      If this is <0 it means no upper limit. */
-  Numeric Uf() const { return muf; }
-
-private:
-  // Molecular species index: 
-  Index mspecies;
-  // Isotopic species index.
-  // If this is equal to the number of isotopes (one more than
-  // allowed) it means all isotopes of this species.
-  Index misotope;
-  // The lower line center frequency in Hz.
-  // If this is <0 it means no lower limit. 
-  Numeric mlf;
-  // The upper line center frequency in Hz:
-  // If this is <0 it means no upper limit. 
-  Numeric muf;
-};
-
-
-/** Output operator for OneTag. 
-
-    \author Stefan Buehler */
-ostream& operator << (ostream& os, const OneTag& ot);
-
-
-/** Contains the available tag groups. Contrary to the Bredbeck
-    definition, tag groups may only consist of tags belonging to the
-    same species. The reason for this is that there is one VMR profile
-    associated with each tag group.
-
-    \author Stefan Buehler */
-typedef  Array< Array<OneTag> > TagGroups;
-
-
-void get_tagindex_for_Strings( 
-              ArrayOfIndex&   tags1_index, 
-        const TagGroups&      tags1, 
-        const ArrayOfString&  tags2_Strings );
-
-void get_tag_group_index_for_tag_group( 
-              Index&         tags1_index, 
-        const TagGroups&      tags1, 
-        const Array<OneTag>&  tags2 );
-
-String get_tag_group_name( const Array<OneTag>& tg );
-
 // Doc header in absorption.cc
 void write_lines_to_stream(ostream& os,
-                           const ArrayOfLineRecord& lines);
+                           const ArrayOfLineRecord& abs_lines);
 
 
-void xsec_species( MatrixView              xsec,
-                   ConstVectorView         f_mono,
-                   ConstVectorView         p_abs,
-                   ConstVectorView         t_abs,           
-                   ConstVectorView         h2o_abs,           
-                   ConstVectorView            vmr,
-                   const ArrayOfLineRecord& lines,
-                   const Index             ind_ls,
-                   const Index             ind_lsn,
-                   const Numeric            cutoff);
+void xsec_species( MatrixView               xsec,
+                   ConstVectorView          f_mono,
+                   ConstVectorView          abs_p,
+                   ConstVectorView          abs_t,           
+                   ConstVectorView          h2o_abs,           
+                   ConstVectorView          vmr,
+                   const ArrayOfLineRecord& abs_lines,
+                   const Index              ind_ls,
+                   const Index              ind_lsn,
+                   const Numeric            cutoff,
+                   const Verbosity&         verbosity );
 
 
 // A helper function for energy conversion:
@@ -1165,13 +1239,13 @@ Numeric wavenumber_to_joule(Numeric e);
 
 void refr_index_BoudourisDryAir (
                     Vector&     refr_index,
-              ConstVectorView   p_abs,
-              ConstVectorView   t_abs );
+              ConstVectorView   abs_p,
+              ConstVectorView   abs_t );
 
 void refr_index_Boudouris (
                     Vector&     refr_index,
-              ConstVectorView   p_abs,
-              ConstVectorView   t_abs,
+              ConstVectorView   abs_p,
+              ConstVectorView   abs_t,
               ConstVectorView   h2o_abs );
 
 //======================================================================
