@@ -1,5 +1,7 @@
-/* Copyright (C) 2000, 2001 Stefan Buehler <sbuehler@uni-bremen.de>,
-                            Axel von Engeln <engeln@uni-bremen.de>
+/* Copyright (C) 2000-2008
+   Stefan Buehler <sbuehler@ltu.se>,
+   Axel von Engeln <engeln@uni-bremen.de>,
+   Carmen Verdes <cverdes@uni-bremen.de>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -27,6 +29,8 @@
   record, if you want to extend the capability of ARTS. These entries
   have to be in the same order, which is assured by check_q_data.
 
+  This is the file from arts-1-0, back-ported to arts-1-1.
+
   \author Axel von Engeln,  C. Verdes,
   \date 2000-08-21 
    */
@@ -135,6 +139,7 @@
 
   <dt> Coefficients: 
   <dd> Generally taken from the new program provided by B. Gamache, version 2003. 
+  </dl>
   Partition functions for species included only in JPL, except for BrO, were
   calculated according to the JPL recommended scheme, and a 3rd order
   polynomial fit was performed (with the idl program mentioned above).
@@ -690,12 +695,9 @@ void spec(Array<SpeciesRecord>::iterator& is,
           String name)
 {
   
-#ifndef NDEBUG
-  {
-    assert( name == is->Name() );
-  }
-#endif
+  assert( name == is->Name() );
 
+  if (name ==  is->Name() ){}
   ii = is->Isotope().begin();
 
   is++;
@@ -707,12 +709,9 @@ void iso(Array<IsotopeRecord>::iterator& ii,
          String name,
          const ArrayOfNumeric& coeff)
 {
-#ifndef NDEBUG
-  {
-    assert( name == ii->Name() );
-  }
-#endif
+  assert( name == ii->Name() );
 
+  if (name ==  ii->Name() ){}
   ii->SetPartitionFctCoeff(coeff);
 
   ii++;
